import sys
sys.path.append('../')
import gi
gi.require_version('Gst', '1.0')
gi.require_version('GstRtspServer', '1.0')
from gi.repository import GObject, Gst, GstRtspServer
from common.is_aarch_64 import is_aarch64
from common.bus_call import bus_call

import pyds	

bitrate=10000000
rtsp_port_num = 8554

def source_to_mux(source_ids, pipeline):

    print("creating nvstreammux \n")
    streammux = Gst.ElementFactory.make("nvstreammux", "muxer")
    if not streammux:
        sys.stderr.write(" Unable to create Streammux\n")
    streammux.set_property('width', 1920)
    streammux.set_property('height', 1080)
    streammux.set_property('batch-size', len(source_ids))
    streammux.set_property('batched-push-timeout', 4000000)

    pipeline.add(streammux)

    for camera in source_ids:
       source_name = "src-elem-%u"%camera
       source = Gst.ElementFactory.make("nvarguscamerasrc", source_name)
       if not source:
           sys.stderr.write(" Unable to create Source\n")
       pipeline.add(source)
       source.set_property('sensor-id', camera)

       sinkpad = streammux.get_request_pad("sink_%u"%camera)
       if not sinkpad:
           sys.stderr.write(" Unable to get the sink pad of streammux \n")
       srcpad = source.get_static_pad("src")
       if not srcpad:
           sys.stderr.write(" Unable to get source pad of caps_vidconvsrc \n")
       srcpad.link(sinkpad)
       source.set_property('bufapi-version', True)

    return streammux

def demuxer_to_sink(number_of_sources, caps, pipeline):

    updsink_port_num = 5400

    print("Creating NvStreamdemux \n")
    streamdemux = Gst.ElementFactory.make("nvstreamdemux", "stream-demuxer")
    if not streamdemux:
        sys.stderr.write(" Unable to create demuxer \n")

    pipeline.add(streamdemux)
    #Link demuxer sink pad to caps 
    caps.link(streamdemux)

    encoder_list = list()
    rtppayload_list = list()
    udpsink_list = list()   

    for i in range(number_of_sources):

       # Make the encoder
       encoder_name = "encoder-%u"%i
       encoder = Gst.ElementFactory.make("nvv4l2h264enc", encoder_name)
       print("Creating H264 Encoder")
       if not encoder:
           sys.stderr.write(" Unable to create encoder")
       encoder.set_property('maxperf-enable',1)
       encoder.set_property('bitrate', bitrate)
       if is_aarch64():
           encoder.set_property('insert-sps-pps', 1)
           encoder.set_property('preset-level', 1)
           encoder.set_property('bufapi-version', 1)
       pipeline.add(encoder)
       encoder_list.append(encoder)
 
       # Make the payload-encode video into RTP packets
       rtppay_name = "rtppay-%u"%i
       rtppay = Gst.ElementFactory.make("rtph264pay", rtppay_name)
       print("Creating H264 rtppay")
       if not rtppay:
           sys.stderr.write(" Unable to create rtppay")
       rtppayload_list.append(rtppay)
       pipeline.add(rtppay)

       # Make the UDP sink
       sink_name = "sink-%u"%i
       sink = Gst.ElementFactory.make("udpsink", sink_name)
       if not sink:
           sys.stderr.write(" Unable to create udpsink")

       sink.set_property('host', '224.224.255.255')
       sink.set_property('port', updsink_port_num + i)
       sink.set_property('async', False)
       sink.set_property('sync', 0)
       updsink_port_num += 1
       udpsink_list.append(sink)
       pipeline.add(sink)

    # we link the demux to the rtppayload
    print("Linking demux to the encoders in the Pipeline \n")
    for i in range(number_of_sources):
       demux_srcpad = streamdemux.get_request_pad("src_%u"%i)
       if not demux_srcpad:
           sys.stderr.write("Unable to get the src pad of streamdemux \n")
       sinkpad = encoder_list[i].get_static_pad("sink")
       if not sinkpad:
           sys.stderr.write(" Unable to get sink pad of encoder \n")
       #demux_srcpad.link(sinkpad)
       sinkpad.link(demux_srcpad)

    # we link the encoders to the rtppayloads
    print("Linking encoders to the rtppayloads in the Pipeline \n")
    for i in range(number_of_sources):
       encoder_list[i].link(rtppayload_list[i])
  

    # we link the rtppayload to the udpsinks
    print("Linking rtppayload to the udpsinks in the Pipeline \n")
    for i in range(number_of_sources):
       rtppayload_list[i].link(udpsink_list[i])

    print("Demuxer successfull !! \n")
    return upsink_list

def main(args):

    source_ids = [0, 1]

    # Standard GStreamer initialization
    GObject.threads_init()
    Gst.init(None)

    # Create gstreamer elements
    # Create Pipeline element that will form a connection of other elements
    print("Creating Pipeline \n ")
    pipeline = Gst.Pipeline()

    if not pipeline:
        sys.stderr.write(" Unable to create Pipeline \n")

    # Creating Source element and attaching to muxer
    streammux = source_to_mux(source_ids, pipeline)
    
    print("Creating video-converter \n")
    nvvidconv_postosd = Gst.ElementFactory.make("nvvideoconvert", "convertor_postosd")
    if not nvvidconv_postosd:
        sys.stderr.write(" Unable to create nvvidconv_postosd \n")
    pipeline.add(nvvidconv_postosd)
    # Create a caps filter
    caps = Gst.ElementFactory.make("capsfilter", "filter")
    caps.set_property("caps", Gst.Caps.from_string("video/x-raw(memory:NVMM), format=NV12, width=3280, height=2464, framerate=21/1"))
    pipeline.add(caps)
    
    streammux.link(nvvidconv_postosd)
    nvvidconv_postosd.link(caps)

    #Link demuxer to all sink elements
    udpsink_list = demuxer_to_sink(len(source_ids), caps, pipeline)

    # create an event loop and feed gstreamer bus mesages to it
    loop = GObject.MainLoop()
    bus = pipeline.get_bus()
    bus.add_signal_watch()
    bus.connect ("message", bus_call, loop)
  
    #Create a new RTSPServer for each element in the udpsink list
    
    for i in range(len(udpsink_list)):
       # Start streaming
       server = GstRtspServer.RTSPServer.new()
       server.props.service = "%d" % rtsp_port_num
       server.attach(None)

       factory = GstRtspServer.RTSPMediaFactory.new()
       factory.set_launch( "( udpsrc name=pay0 port=%d buffer-size=524288 caps=\"application/x-rtp, media=video, clock-rate=90000, encoding-name=(string)%s, payload=96 \" )" % (updsink_port_num + i, "H264"))
       factory.set_shared(True)
       server.get_mount_points().add_factory("/ds-test", factory)
       print("\n *** DeepStream: Launched RTSP Streaming at rtsp://localhost:%d/ds-test ***\n\n" % rtsp_port_num + i)



    # start play back and listen to events
    print("Starting pipeline \n")
    pipeline.set_state(Gst.State.PLAYING)
    try:
        loop.run()
    except:
        pass
    # cleanup
    pipeline.set_state(Gst.State.NULL)
if __name__ == '__main__':
    sys.exit(main(sys.argv))
